package com.biganto.visual.roompark.presentation.screen.auth

import android.view.View
import butterknife.BindView
import com.biganto.visual.roompark.R
import com.biganto.visual.roompark.base.RoomParkApplication
import com.biganto.visual.roompark.base.RoomParkMainActivity
import com.biganto.visual.roompark.conductor.BigantoBaseController
import com.biganto.visual.roompark.presentation.screen.home.home_routing.HomeBottomNavigationController
import com.bluelinelabs.conductor.RouterTransaction
import com.google.android.material.button.MaterialButton
import com.google.android.material.snackbar.Snackbar
import com.google.android.material.textfield.TextInputLayout
import com.jakewharton.rxbinding3.view.clicks
import io.reactivex.Observable
import timber.log.Timber
import javax.inject.Inject

/**
 * Created by Vladislav Bogdashkin on 30.09.2019.
 */

class AuthScreenController  :
    BigantoBaseController<AuthScreenViewState
        , AuthScreen
        , AuthScreenPresenter>()
    , AuthScreen {
    override fun onViewBound(v: View) {
    }

    @BindView(R.id.login_text_input) lateinit var loginInput:TextInputLayout
    @BindView(R.id.password_text_input) lateinit var pwdInput:TextInputLayout
    @BindView(R.id.sign_in_button) lateinit var signInButton:MaterialButton





    override fun tryAuth(): Observable<AuthInputModel> =
        signInButton.clicks()
            .map<AuthInputModel>{ AuthInputModel(loginInput.editText?.text?.toString()?:""
                ,pwdInput.editText?.text?.toString()?:"")
            }

    override fun injectDependencies() {
        getComponent()
    }


    @Inject
    override lateinit var injectedPresenter: AuthScreenPresenter


//    @Inject
//    lateinit var snacky:ISnackBarProvider

    lateinit var rpActivity: RoomParkMainActivity

    fun getComponent() = DaggerAuthScreenComponent
        .factory()
        .create(RoomParkApplication.component,activity as RoomParkMainActivity)
        .inject(this)

//    @Inject
//    lateinit var ac: RoomParkMainActivity



    override fun render(viewState: AuthScreenViewState) {
        super.render(viewState)
        Timber.d("Render state $viewState")
        when(viewState){
            is AuthScreenViewState.Idle -> render(viewState)
            is AuthScreenViewState.Authorization -> render(viewState)
            is AuthScreenViewState.SignedIn -> render(viewState)
            is AuthScreenViewState.SignInError -> render(viewState)
            is AuthScreenViewState.SomeError -> render(viewState)

        }
    }

    private fun render(viewState: AuthScreenViewState.Idle){

    }

    private fun render(viewState: AuthScreenViewState.Authorization){
//        toolBar.hideAll()
        signInButton.isEnabled = false
//        snacky.showSnackBar("lul")
    }

    private fun render(viewState: AuthScreenViewState.SignedIn){
        router.setRoot(RouterTransaction.with(HomeBottomNavigationController()))
//        snacky.showSnackBar("lul")
    }

    private fun render(viewState: AuthScreenViewState.SignInError){
        viewState.exception.stringId?.let { snackbar.showSnackBar(it) }
        viewState.exception.message?.let { snackbar.showSnackBar(it, Snackbar.LENGTH_LONG) }
        signInButton.isEnabled=true
    }

    private fun render(viewState: AuthScreenViewState.SomeError){
        viewState.exception.stringId?.let { snackbar.showSnackBar(it) }
        viewState.exception.message?.let { snackbar.showSnackBar(it, Snackbar.LENGTH_LONG) }
    }


    override fun getLayoutId(): Int = R.layout.authentication_screen


}