package com.biganto.visual.roompark.presentation.screen.auth

import com.biganto.visual.roompark.R
import com.biganto.visual.roompark.conductor.BigantoBasePresenter
import com.biganto.visual.roompark.data.service.network.NoNetworkException
import com.biganto.visual.roompark.domain.custom_exception.CustomApiException
import com.biganto.visual.roompark.domain.interactor.AuthInteractor
import com.biganto.visual.roompark.util.monades.ExceptionString
import io.reactivex.android.schedulers.AndroidSchedulers
import io.reactivex.schedulers.Schedulers
import timber.log.Timber
import javax.inject.Inject

/**
 * Created by Vladislav Bogdashkin on 30.09.2019.
 */


class AuthScreenPresenter @Inject constructor(
    private val interactor: AuthInteractor
)
    : BigantoBasePresenter<AuthScreen, AuthScreenViewState>() {

    private fun parseError(t: Throwable): AuthScreenViewState =
        when (t) {
            is CustomApiException -> onCodeReturn(e = t)
            is NoNetworkException -> onNoNetwork(t)
            else -> onRandomError(t)
        }


    private fun onRandomError(t: Throwable): AuthScreenViewState =
        AuthScreenViewState.SomeError(
            ExceptionString(R.string.unknown_error, null)
        )

    private fun onNoNetwork(e: NoNetworkException): AuthScreenViewState =
        AuthScreenViewState.SomeError(
            ExceptionString(R.string.no_network_error, null)
        )

    private fun onCodeReturn(e: CustomApiException): AuthScreenViewState =
        when (e.code) {
            else -> AuthScreenViewState.SignInError(ExceptionString(e))
        }


    override fun bindIntents() {

        val onAuth = intent(AuthScreen::tryAuth)
            .flatMap<AuthScreenViewState> { model ->
                interactor.signIn(model.login, model.pwd)
                    .doOnNext { Timber.d("auth returned $it") }
                    .map { it }

                    .map { AuthScreenViewState.SignedIn() }
            }
//            .startWith(Observable.just(AuthScreenViewState.Authorization()))


        val state = restoreStateObservable
            .mergeWith(onAuth)
            .doOnError { Timber.e(it) }
            .onErrorReturn(::parseError)
            .subscribeOn(Schedulers.single())
            .observeOn(AndroidSchedulers.mainThread())

        subscribeViewState(state.cast(AuthScreenViewState::class.java), AuthScreen::render)
    }

}